@@ -24,7 +24,6 @@
 #ifdef ENABLE_FP4
 #include <cuda_fp4.h>
 #endif
-
 #include <cuda_runtime_api.h>
 
 #include <array>
@@ -86,6 +85,54 @@ struct LoraParams {
 };
 
 namespace cutlass_kernels {
+static inline size_t pad_to_multiple_of_16(size_t const& input) {
+  static constexpr int ALIGNMENT = 16;
+  return ALIGNMENT * ((input + ALIGNMENT - 1) / ALIGNMENT);
+}
+
+class CubKeyValueSorter {
+ public:
+  CubKeyValueSorter();
+
+  CubKeyValueSorter(int const num_experts_per_node);
+
+  void updateNumExperts(int const num_experts_per_node);
+
+  static size_t getWorkspaceSize(size_t const num_key_value_pairs, int const num_experts_per_node);
+
+  void run(void* workspace, size_t const workspace_size, int const* keys_in, int* keys_out,
+           int const* values_in, int* values_out, size_t const num_key_value_pairs,
+           cudaStream_t stream);
+
+ private:
+  static int expertsToBits(int experts);
+  int num_experts_;
+  int num_bits_;
+};
+
+struct ActivationParams {
+  ActivationType activation_type;
+  float const* swiglu_alpha = nullptr;
+  float const* swiglu_beta = nullptr;
+  float const* swiglu_limit = nullptr;
+
+  explicit ActivationParams(ActivationType activation_type) : activation_type(activation_type) {
+    TLLM_CHECK_WITH_INFO(
+        activation_type != ActivationType::SwigluBias,
+        "SwigluBias is not supported in ActivationParams without swiglu_alpha and swiglu_beta");
+  }
+
+  ActivationParams(ActivationType activation_type, float const* swiglu_alpha,
+                   float const* swiglu_beta, float const* swiglu_limit)
+      : activation_type(activation_type),
+        swiglu_alpha(swiglu_alpha),
+        swiglu_beta(swiglu_beta),
+        swiglu_limit(swiglu_limit) {}
+
+  // TODO Port everything properly and get rid of these implicit conversions
+  operator ActivationType() const { return activation_type; }
+};
+
 /**
  * \brief Describes what parallelism mode the MoE is using
  *
@@ -384,7 +431,7 @@ class CutlassMoeFCRunnerInterface {
   virtual void runMoe(void const* input_activations, void const* input_sf,
                       int const* token_selected_experts, float const* token_final_scales,
                       void const* fc1_expert_weights, void const* fc1_expert_biases,
-                      ActivationType fc1_activation_type, void const* fc2_expert_weights,
+                      ActivationParams fc1_activation_type, void const* fc2_expert_weights,
                       void const* fc2_expert_biases, QuantParams quant_params,
                       int64_t const num_rows, int64_t const hidden_size, int64_t const inter_size,
                       int const num_experts, int const experts_per_token, char* workspace_ptr,
@@ -406,7 +453,7 @@ class CutlassMoeFCRunnerInterface {
                      QuantParams quant_params, int64_t const num_rows,
                      int64_t const expanded_num_rows, int64_t const hidden_size,
                      int64_t const inter_size, int const num_experts_per_node,
-                     ActivationType fc1_activation_type, float const** alpha_scale_ptr_array,
+                     ActivationParams fc1_activation_type, float const** alpha_scale_ptr_array,
                      bool bias_is_broadcast, bool use_deepseek_fp8_block_scale, cudaStream_t stream,
                      cutlass_extensions::CutlassGemmConfig config, bool min_latency_mode,
                      int* num_active_experts_per, int* active_expert_global_ids,
@@ -479,6 +526,15 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
       tensorrt_llm::kernels::fp8_blockscale_gemm::CutlassFp8BlockScaleGemmRunnerInterface;
   using ScaleBiasType = BackBoneType;
   using Self = CutlassMoeFCRunner<T, WeightType, OutputType, InputType, BackBoneType>;
+
+#if defined(ENABLE_BF16)
+  static constexpr bool use_wfp4a16 = std::is_same_v<WeightType, __nv_fp4_e2m1> &&
+                                      (std::is_same_v<T, half> || std::is_same_v<T, __nv_bfloat16>);
+#else
+  static constexpr bool use_wfp4a16 =
+      std::is_same_v<WeightType, __nv_fp4_e2m1> && std::is_same_v<T, half>;
+#endif
+
 #if defined(ENABLE_FP8)
   static constexpr bool use_fp8 =
       (std::is_same_v<T, __nv_fp8_e4m3> || std::is_same_v<T, __nv_fp8_e5m2>) &&
@@ -493,6 +549,7 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
   static constexpr bool use_fp8 = false;
   static constexpr bool use_w4afp8 = false;
 #endif
+  static constexpr bool use_w4_groupwise = use_w4afp8 || use_wfp4a16;
 #if defined(ENABLE_FP4)
   static constexpr bool act_fp4 = std::is_same_v<T, __nv_fp4_e2m1>;
   static constexpr bool weight_fp4 = std::is_same_v<WeightType, __nv_fp4_e2m1>;
@@ -552,7 +609,7 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
   void runMoe(void const* input_activations, void const* input_sf,
               int const* token_selected_experts, float const* token_final_scales,
               void const* fc1_expert_weights, void const* fc1_expert_biases,
-              ActivationType fc1_activation_type, void const* fc2_expert_weights,
+              ActivationParams fc1_activation_type, void const* fc2_expert_weights,
               void const* fc2_expert_biases, QuantParams quant_params, int64_t const num_rows,
               int64_t const hidden_size, int64_t const inter_size, int const num_experts,
               int const experts_per_token, char* workspace_ptr, void* final_output,
@@ -563,27 +620,30 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
               cudaStream_t stream) override;
 
   // We make these GEMM1 & GEMM2 static because they need to be stateless for the profiler to work
-  static void gemm1(
-      MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>& gemm_runner,
-      // This argument must not be null if fp8 block scaling is being used.
-      // The gemm_runner will be ignored in that case. NOTE: it would
-      // be great if we could consolidate gemm_runner and fp8_blockscale_gemm_runner.
-      // For now, they don't share the same interface, so we just use two separate
-      // arguments.
-      DeepSeekBlockScaleGemmRunner* fp8_blockscale_gemm_runner, T const* const input,
-      T* const output, void* const intermediate_result,
-      int64_t const* const expert_first_token_offset,
-      TmaWarpSpecializedGroupedGemmInput const tma_ws_input_template,
-      WeightType const* const fc1_expert_weights, ScaleBiasType const* const fc1_expert_biases,
-      int64_t const* const num_valid_tokens_ptr, ScaleBiasType const* const fc1_int_scales,
-      float const* const fc1_fp8_dequant, float const* const fc2_fp8_quant,
-      TmaWarpSpecializedGroupedGemmInput::ElementSF const* fc1_fp4_act_flat,
-      TmaWarpSpecializedGroupedGemmInput::ElementSF* fc2_fp4_act_flat, QuantParams quant_params,
-      int64_t const num_rows, int64_t const expanded_num_rows, int64_t const hidden_size,
-      int64_t const inter_size, int const num_experts_per_node, ActivationType fc1_activation_type,
-      float const** alpha_scale_ptr_array, bool bias_is_broadcast, cudaStream_t stream,
-      cutlass_extensions::CutlassGemmConfig config, bool min_latency_mode,
-      int* num_active_experts_per, int* active_expert_global_ids, bool enable_pdl);
+  static void gemm1(MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>& gemm_runner,
+                    // This argument must not be null if fp8 block scaling is being used.
+                    // The gemm_runner will be ignored in that case. NOTE: it would
+                    // be great if we could consolidate gemm_runner and fp8_blockscale_gemm_runner.
+                    // For now, they don't share the same interface, so we just use two separate
+                    // arguments.
+                    DeepSeekBlockScaleGemmRunner* fp8_blockscale_gemm_runner, T const* const input,
+                    T* const output, void* const intermediate_result,
+                    int64_t const* const expert_first_token_offset,
+                    TmaWarpSpecializedGroupedGemmInput const tma_ws_input_template,
+                    WeightType const* const fc1_expert_weights,
+                    ScaleBiasType const* const fc1_expert_biases,
+                    int64_t const* const num_valid_tokens_ptr,
+                    ScaleBiasType const* const fc1_int_scales, float const* const fc1_fp8_dequant,
+                    float const* const fc2_fp8_quant,
+                    TmaWarpSpecializedGroupedGemmInput::ElementSF const* fc1_fp4_act_flat,
+                    TmaWarpSpecializedGroupedGemmInput::ElementSF* fc2_fp4_act_flat,
+                    QuantParams quant_params, int64_t const num_rows,
+                    int64_t const expanded_num_rows, int64_t const hidden_size,
+                    int64_t const inter_size, int const num_experts_per_node,
+                    ActivationParams fc1_activation_type, float const** alpha_scale_ptr_array,
+                    bool bias_is_broadcast, cudaStream_t stream,
+                    cutlass_extensions::CutlassGemmConfig config, bool min_latency_mode,
+                    int* num_active_experts_per, int* active_expert_global_ids, bool enable_pdl);
 
   static void gemm2(
       MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>& gemm_runner,
@@ -617,7 +677,7 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
              TmaWarpSpecializedGroupedGemmInput::ElementSF* fc2_fp4_act_flat,
              QuantParams quant_params, int64_t const num_rows, int64_t const expanded_num_rows,
              int64_t const hidden_size, int64_t const inter_size, int const num_experts_per_node,
-             ActivationType fc1_activation_type, float const** alpha_scale_ptr_array,
+             ActivationParams fc1_activation_type, float const** alpha_scale_ptr_array,
              bool bias_is_broadcast, bool use_deepseek_fp8_block_scale, cudaStream_t stream,
              cutlass_extensions::CutlassGemmConfig config, bool min_latency_mode,
              int* num_active_experts_per, int* active_expert_global_ids, bool enable_pdl) override {
@@ -724,7 +784,7 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
  private:
   std::pair<TmaWarpSpecializedGroupedGemmInput, TmaWarpSpecializedGroupedGemmInput>
   setupTmaWarpSpecializedInputs(int64_t num_rows, int64_t expanded_num_rows,
-                                ActivationType fc1_activation_type, int64_t hidden_size,
+                                ActivationParams fc1_activation_type, int64_t hidden_size,
                                 int64_t inter_size, int64_t num_experts_per_node,
                                 void const* input_activations_void,
                                 TmaWarpSpecializedGroupedGemmInput::ElementSF const* input_sf,
@@ -781,7 +841,7 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
 
   bool mayHaveFinalizeFused() const {
     return moe_gemm_runner_.supportsTmaWarpSpecialized() && moe_gemm_runner_.getSM() == 90 &&
-           !use_deterministic_hopper_reduce_ && !use_w4afp8;
+           !use_deterministic_hopper_reduce_ && !use_w4_groupwise;
   }
 
   // TODO: This should eventually take the quant params to give more flexibility
@@ -816,7 +876,7 @@ class CutlassMoeFCRunner : public CutlassMoeFCRunnerInterface {
                             float const* const fc2_fp8_quant, int64_t const num_rows,
                             int64_t const expanded_num_rows, int64_t const hidden_size,
                             int64_t const inter_size, int const num_experts_per_node,
-                            ActivationType fc1_activation_type, QuantParams& quant_params,
+                            ActivationParams fc1_activation_type, QuantParams& quant_params,
                             bool enable_pdl, cudaStream_t stream);
 
   static void BlockScaleFC2(
