@@ -22,7 +22,7 @@
 
 namespace tensorrt_llm {
 
-enum class FP4QuantizationSFLayout {
+enum class QuantizationSFLayout {
   // Block scale factors are stored in swizzled layout for cutlass FP4 kernel. Scale factor
   // blocks are organized in 512-byte blocks in global memory, with each block having 128x4 FP8
   // values. The SF matrix dimensions are therefore padded - rows to the nearest multiple of 128 and
@@ -32,8 +32,6 @@ enum class FP4QuantizationSFLayout {
   // For a scale factor row 'i', it maps to data block row: (i % 4) * 32 + (i / 4)
   // Column 'j' in the scale factor block corresponds to scaling the j-th block in the data tensor.
   SWIZZLED_128x4,
-
-  // Similar to SWIZZLED_128x4, but with 8x4 scale factor blocks.
   SWIZZLED_8x4,
 
   // Block scale factors are stored in linear layout (row-major). This is used in some trtllm-gen
@@ -51,14 +49,14 @@ enum class BlockScaleQuantizationType {
 #define PadUpFn(X, Y) ((X + Y - 1) / (Y) * (Y))
 
 // totalCloumn should be in SFMatrix, not activation Matrix, so no sfVecSize needed.
-inline int computeFP4SwizzledLayoutSFSize(int totalRow, int totalColumn, int rowSize = 128) {
+inline int64_t computeSwizzledLayoutSFSize(int totalRow, int totalColumn, int rowSize = 128) {
   int paddedRow = PadUpFn(totalRow, rowSize);
   int paddedColumn = PadUpFn(totalColumn, 4);
-  return paddedRow * paddedColumn;
+  return static_cast<int64_t>(paddedRow) * paddedColumn;
 }
 
-inline int computeFP4LinearLayoutSFSize(int totalRow, int totalColumn) {
-  return totalRow * totalColumn;
+inline int64_t computeLinearLayoutSFSize(int totalRow, int totalColumn) {
+  return static_cast<int64_t>(totalRow) * totalColumn;
 }
 
 namespace kernels {
@@ -73,32 +71,23 @@ void invokePerTokenQuantization(QuantT* dst, T const* src, int64_t const numRows
                                 float* sumPtr, tensorrt_llm::common::QuantMode quantMode,
                                 cudaStream_t stream = 0);
 
-template <typename T, int SF_VEC_SIZE = 16>
-void invokeFP4Quantization(int m, int n, T const* input, float const* globalScale, int64_t* output,
-                           int32_t* SFOuput, bool useUE8M0, FP4QuantizationSFLayout layout,
-                           int multiProcessorCount, bool enable_pdl = false,
-                           cudaStream_t stream = 0);
+template <typename T, int SF_VEC_SIZE>
+void invokeFP4Quantization(int b, int m, int n, T const* input, float const* globalScale,
+                           int64_t* output, int32_t* SFOuput, bool useUE8M0,
+                           QuantizationSFLayout layout, int multiProcessorCount,
+                           bool enable_pdl = false, cudaStream_t stream = 0);
 
-template <typename T, int SF_VEC_SIZE = 16>
-void invokeBatchedFP4Quantization(
-    int b, int m, int n, T const* input, float const* globalScale, int64_t* output,
-    int32_t* SFOuput, bool useUE8M0, int multiProcessorCount,
-    FP4QuantizationSFLayout layout = FP4QuantizationSFLayout::SWIZZLED_128x4,
-    bool enable_pdl = false, cudaStream_t stream = 0);
+void invokeBlockScaleInterleave(int b, int m, int m_padded, int n, int n_padded,
+                                uint8_t const* SFIn, uint8_t* SFOutput, int multiProcessorCount,
+                                cudaStream_t stream = 0);
 
-void invokeNVFP4BlockScaleInterleave(int b, int m, int m_padded, int n, int n_padded,
-                                     uint8_t const* SFIn, uint8_t* SFOutput,
-                                     int multiProcessorCount, cudaStream_t stream = 0);
-
-void invokeNVFP4BlockScaleInterleaveReverse(int b, int m, int n, uint8_t const* SFIn,
-                                            uint8_t* SFOutput, int multiProcessorCount,
-                                            cudaStream_t stream = 0);
+void invokeBlockScaleInterleaveReverse(int b, int m, int n, uint8_t const* SFIn, uint8_t* SFOutput,
+                                       int multiProcessorCount, cudaStream_t stream = 0);
 
 template <typename T>
 void invokeMxFP8Quantization(int b, int m, int n, int padded_n, T const* input, int64_t* output,
-                             int32_t* SFOuput, FP4QuantizationSFLayout layout,
-                             int multiProcessorCount, bool enable_pdl = false,
-                             cudaStream_t stream = 0);
+                             int32_t* SFOuput, QuantizationSFLayout layout, int multiProcessorCount,
+                             bool enable_pdl = false, cudaStream_t stream = 0);
 
 }  // namespace kernels
 }  // namespace tensorrt_llm
