@@ -145,23 +145,39 @@ FLASHINFER_EXT_MODULE_INIT_EXPAND(TORCH_EXTENSION_NAME)
 
 // Should not be used together with _DISPATCH_SF_CASE_FP8_E8M0
 #ifdef FLASHINFER_ENABLE_FP4_E2M1
+#if (__CUDACC_VER_MAJOR__ * 10000 + __CUDACC_VER_MINOR__ * 100 >= 120800)
 #define _DISPATCH_CASE_FP4_E2M1(c_type, ...) \
   case at::ScalarType::Byte: {               \
     using c_type = __nv_fp4_e2m1;            \
     return __VA_ARGS__();                    \
   }
 #else
+#define _DISPATCH_CASE_FP4_E2M1(c_type, ...)                               \
+  case at::ScalarType::Byte: {                                             \
+    static_assert(false, "FP4 E2M1 support requires CUDA 12.8 or newer."); \
+    break;                                                                 \
+  }
+#endif
+#else
 #define _DISPATCH_CASE_FP4_E2M1(c_type, ...)
 #endif
 
 // Should not be used together with _DISPATCH_CASE_FP4_E2M1
 #ifdef FLASHINFER_ENABLE_FP8_E8M0
+#if (__CUDACC_VER_MAJOR__ * 10000 + __CUDACC_VER_MINOR__ * 100 >= 120800)
 #define _DISPATCH_SF_CASE_FP8_E8M0(c_type, ...) \
   case at::ScalarType::Byte: {                  \
     using c_type = __nv_fp8_e8m0;               \
     return __VA_ARGS__();                       \
   }
 #else
+#define _DISPATCH_SF_CASE_FP8_E8M0(c_type, ...)                            \
+  case at::ScalarType::Byte: {                                             \
+    static_assert(false, "FP8 E8M0 support requires CUDA 12.8 or newer."); \
+    break;                                                                 \
+  }
+#endif
+#else
 #define _DISPATCH_SF_CASE_FP8_E8M0(c_type, ...)
 #endif
 
@@ -298,6 +314,10 @@ inline constexpr uint32_t pack_u16(uint16_t a, uint16_t b) {
   CHECK_CONTIGUOUS(x)
 #define CHECK_INPUT_TYPE(x, st) \
   TORCH_CHECK(x.scalar_type() == st, "Inconsistency of Tensor type: " #x)
+#define CHECK_INPUT_AND_TYPE(x, st) \
+  CHECK_CUDA(x);                    \
+  CHECK_CONTIGUOUS(x);              \
+  CHECK_INPUT_TYPE(x, st)
 #define CHECK_LAST_DIM_CONTIGUOUS_INPUT(x) \
   CHECK_CUDA(x);                           \
   CHECK_LAST_DIM_CONTIGUOUS(x)
