@@ -20,6 +20,7 @@ from .core import (
     convert_to_block_layout,
     cutlass_fused_moe,
     gen_cutlass_fused_moe_sm100_module,
+    gen_cutlass_fused_moe_sm90_module,
     reorder_rows_for_gated_act_gemm,
     trtllm_fp4_block_scale_moe,
     trtllm_fp4_block_scale_routed_moe,
@@ -33,6 +34,7 @@ __all__ = [
     "convert_to_block_layout",
     "cutlass_fused_moe",
     "gen_cutlass_fused_moe_sm100_module",
+    "gen_cutlass_fused_moe_sm90_module",
     "reorder_rows_for_gated_act_gemm",
     "trtllm_fp4_block_scale_moe",
     "trtllm_fp8_block_scale_moe",
