@@ -33,7 +33,7 @@
 
 namespace moe::dev {
 
-#define CHECK_CUDA(cmd)                                                                      \
+#define CHECK_CUDA_ERROR(cmd)                                                                \
   do {                                                                                       \
     cudaError_t e = cmd;                                                                     \
     if (e != cudaSuccess) {                                                                  \
@@ -46,36 +46,34 @@ namespace moe::dev {
 
 #define LAUNCH_ESC(...) __VA_ARGS__
 
-#define LAUNCH_PDL(data, coopLaunch, types, kernel, numBlocks, numThreads, smemSize, stream)    \
-  cudaLaunchConfig_t config{};                                                                  \
-  config.gridDim = numBlocks;                                                                   \
-  config.blockDim = numThreads;                                                                 \
-  config.dynamicSmemBytes = smemSize;                                                           \
-  config.stream = (cudaStream_t)stream;                                                         \
-                                                                                                \
-  cudaLaunchAttribute attributes[2] = {};                                                       \
-  attributes[0].id = cudaLaunchAttributeProgrammaticStreamSerialization;                        \
-  attributes[0].val.programmaticStreamSerializationAllowed = int(data.mUsePdl);                 \
-  attributes[1].id = cudaLaunchAttributeCooperative;                                            \
-  attributes[1].val.cooperative = int(coopLaunch);                                              \
-  config.attrs = attributes;                                                                    \
-  config.numAttrs = 2;                                                                          \
-  if (data.mUsePdl) {                                                                           \
-    auto params = KernelParams<types, true>::setKernelParams(data);                             \
-    auto kernelTyped = kernel<KernelParams<types, true>>;                                       \
-    if (smemSize > 48 * 1024) {                                                                 \
-      CHECK_CUDA(cudaFuncSetAttribute(kernelTyped, cudaFuncAttributeMaxDynamicSharedMemorySize, \
-                                      smemSize));                                               \
-    }                                                                                           \
-    CHECK_CUDA(cudaLaunchKernelEx(&config, kernelTyped, params));                               \
-  } else {                                                                                      \
-    auto params = KernelParams<types, false>::setKernelParams(data);                            \
-    auto kernelTyped = kernel<KernelParams<types, false>>;                                      \
-    if (smemSize > 48 * 1024) {                                                                 \
-      CHECK_CUDA(cudaFuncSetAttribute(kernelTyped, cudaFuncAttributeMaxDynamicSharedMemorySize, \
-                                      smemSize));                                               \
-    }                                                                                           \
-    CHECK_CUDA(cudaLaunchKernelEx(&config, kernelTyped, params));                               \
+#define LAUNCH_PDL(data, coopLaunch, types, kernel, numBlocks, numThreads, smemSize, stream) \
+  cudaLaunchConfig_t config{};                                                               \
+  config.gridDim = numBlocks;                                                                \
+  config.blockDim = numThreads;                                                              \
+  config.dynamicSmemBytes = smemSize;                                                        \
+  config.stream = (cudaStream_t)stream;                                                      \
+                                                                                             \
+  cudaLaunchAttribute attributes[2] = {};                                                    \
+  attributes[0].id = cudaLaunchAttributeProgrammaticStreamSerialization;                     \
+  attributes[0].val.programmaticStreamSerializationAllowed = int(data.mUsePdl);              \
+  attributes[1].id = cudaLaunchAttributeCooperative;                                         \
+  attributes[1].val.cooperative = int(coopLaunch);                                           \
+  config.attrs = attributes;                                                                 \
+  config.numAttrs = 2;                                                                       \
+  if (data.mUsePdl) {                                                                        \
+    auto params = KernelParams<types, true>::setKernelParams(data);                          \
+    auto kernelTyped = kernel<KernelParams<types, true>>;                                    \
+    if (smemSize > 48 * 1024)                                                                \
+      CHECK_CUDA_ERROR(cudaFuncSetAttribute(                                                 \
+          kernelTyped, cudaFuncAttributeMaxDynamicSharedMemorySize, smemSize));              \
+    CHECK_CUDA_ERROR(cudaLaunchKernelEx(&config, kernelTyped, params));                      \
+  } else {                                                                                   \
+    auto params = KernelParams<types, false>::setKernelParams(data);                         \
+    auto kernelTyped = kernel<KernelParams<types, false>>;                                   \
+    if (smemSize > 48 * 1024)                                                                \
+      CHECK_CUDA_ERROR(cudaFuncSetAttribute(                                                 \
+          kernelTyped, cudaFuncAttributeMaxDynamicSharedMemorySize, smemSize));              \
+    CHECK_CUDA_ERROR(cudaLaunchKernelEx(&config, kernelTyped, params));                      \
   }
 
 #define LAUNCH(data, kernel, numBlocks, numThreads, smemSize, stream)                              \
@@ -361,7 +359,10 @@ struct Data {
   int32_t numTokens;
   int32_t numExperts;
   int32_t topK;
+  // Hidden dimension output of MoE block. It is not padded.
   int32_t hiddenDim;
+  // Hidden dimension output of FC2. It might be padded.
+  int32_t hiddenDimPadded;
   int32_t const* totalNumPaddedTokens;
 };
 
@@ -381,6 +382,7 @@ struct KernelParams {
   int32_t* expandedIdxToPermutedIdx;
 
   int32_t hiddenDim;
+  int32_t hiddenDimPadded;
   int32_t numTokens;
   int32_t numExperts;
   int32_t topK;
@@ -398,6 +400,7 @@ struct KernelParams {
     params.expandedIdxToPermutedIdx = data.expandedIdxToPermutedIdx;
 
     params.hiddenDim = data.hiddenDim;
+    params.hiddenDimPadded = data.hiddenDimPadded;
     params.numTokens = data.numTokens;
     params.numExperts = data.numExperts;
     params.topK = data.topK;
