@@ -5,7 +5,6 @@ import torch
 from conftest import VARLEN_INDPTR_PARAMS
 
 import flashinfer
-import flashinfer.triton
 from flashinfer.utils import is_sm100a_supported
 
 
@@ -244,6 +243,89 @@ def test_blackwell_cutlass_varlen(
     torch.testing.assert_close(lse, lse_ref, rtol=1e-3, atol=1e-3)
 
 
+@pytest.mark.parametrize("qo_indptr_list", [[0, 10, 20, 30, 40, 50, 60, 100]])
+@pytest.mark.parametrize("kv_indptr_list", [[0, 50, 50, 50, 50, 50, 50, 50]])
+@pytest.mark.parametrize("num_qo_heads", [32])
+@pytest.mark.parametrize("num_kv_heads", [8, 32])
+@pytest.mark.parametrize("head_dim_qk", [192, 128])
+@pytest.mark.parametrize("head_dim_vo", [128])
+@pytest.mark.parametrize("sm_scale", [1.0 / math.sqrt(128)])
+@pytest.mark.parametrize("dtype", [torch.half, torch.bfloat16])
+def test_blackwell_cutlass_qo_kv_varlen(
+    qo_indptr_list,
+    kv_indptr_list,
+    num_qo_heads,
+    num_kv_heads,
+    head_dim_qk,
+    head_dim_vo,
+    sm_scale,
+    dtype,
+):
+    causal = False
+    if not is_sm100a_supported(torch.device("cuda")):
+        pytest.skip("SM100A is not supported on this device")
+    torch.manual_seed(42)
+    q = torch.randn(
+        qo_indptr_list[-1],
+        num_qo_heads,
+        head_dim_qk,
+        dtype=dtype,
+        device="cuda",
+    )
+    k = torch.randn(
+        kv_indptr_list[-1],
+        num_kv_heads,
+        head_dim_qk,
+        dtype=dtype,
+        device="cuda",
+    )
+    v = torch.randn(
+        kv_indptr_list[-1],
+        num_kv_heads,
+        head_dim_vo,
+        dtype=dtype,
+        device="cuda",
+    )
+
+    qo_indptr = torch.tensor(qo_indptr_list, device="cuda", dtype=torch.int32)
+    kv_indptr = torch.tensor(kv_indptr_list, device="cuda", dtype=torch.int32)
+
+    wrapper = flashinfer.prefill.BatchPrefillWithRaggedKVCacheWrapper(
+        torch.empty(128 * 1024 * 1024, device="cuda", dtype=torch.uint8),
+        kv_layout="NHD",
+        backend="cutlass",
+    )
+
+    wrapper.plan(
+        qo_indptr,
+        kv_indptr,
+        num_qo_heads,
+        num_kv_heads,
+        head_dim_qk,
+        head_dim_vo=head_dim_vo,
+        causal=causal,
+        sm_scale=sm_scale,
+        q_data_type=dtype,
+        kv_data_type=dtype,
+    )
+    o, lse = wrapper.run(q, k, v, return_lse=True)
+
+    gqa_group_ratio = num_qo_heads // num_kv_heads
+    k_repeated = torch.repeat_interleave(k, gqa_group_ratio, dim=1)
+    v_repeated = torch.repeat_interleave(v, gqa_group_ratio, dim=1)
+
+    o_ref, lse_ref = attention_varlen_ref(
+        q, k_repeated, v_repeated, qo_indptr, kv_indptr, causal, sm_scale
+    )
+
+    if dtype == torch.half:
+        torch.testing.assert_close(o[10:60], o_ref[10:60], rtol=1e-3, atol=1e-3)
+    else:
+        torch.testing.assert_close(o[10:60], o_ref[10:60], rtol=1e-2, atol=1e-2)
+
+    torch.testing.assert_close(lse, lse_ref, rtol=1e-3, atol=1e-3)
+
+
 if __name__ == "__main__":
     test_blackwell_cutlass_fmha(
         9,
@@ -268,3 +350,14 @@ if __name__ == "__main__":
         True,
         torch.bfloat16,
     )
+
+    test_blackwell_cutlass_qo_kv_varlen(
+        [0, 10, 20, 30, 40, 50, 60, 100],
+        [0, 50, 50, 50, 50, 50, 50, 50],
+        32,
+        8,
+        128,
+        128,
+        1,
+        torch.bfloat16,
+    )
