@@ -33,7 +33,7 @@ from flashinfer import (
     reorder_rows_for_gated_act_gemm,
     shuffle_matrix_a,
 )
-from flashinfer.fp4_quantization import nvfp4_block_scale_interleave
+from flashinfer.fp4_quantization import block_scale_interleave
 from flashinfer.fused_moe import (
     WeightLayout,
     convert_to_block_layout,
@@ -357,19 +357,11 @@ class FP4Moe(Moe):
                 hidden_states, is_swizzling
             )
             hidden_states_scale = hidden_states_scale.view(torch.float8_e4m3fn).reshape(
-                -1
-            )
-            print(
-                f"hidden_states.shape: {hidden_states_quant.shape}, dtype: {hidden_states_quant.dtype}"
-            )
-            print(
-                f"hidden_states_scale.shape: {hidden_states_scale.shape}, dtype: {hidden_states_scale.dtype}"
+                *hidden_states.shape[:-1], -1
             )
             return {
                 "hidden_states": hidden_states_quant,
-                "hidden_states_scale": hidden_states_scale.view(
-                    torch.float8_e4m3fn
-                ).reshape(-1),
+                "hidden_states_scale": hidden_states_scale,
             }
         elif self.quant_mode == QuantMode.FP4_NVFP4_NVFP4:
             """Quantize hidden states to NvFP4 format using pre-computed global scale."""
@@ -380,12 +372,13 @@ class FP4Moe(Moe):
             ) = quant_fp4(
                 hidden_states, hidden_states_scale_global, False, is_swizzling
             )
+            hidden_states_scale_fp4_bytes = hidden_states_scale_fp4_bytes.view(
+                torch.float8_e4m3fn
+            ).reshape(*hidden_states.shape[:-1], -1)
 
             return {
                 "hidden_states": hidden_states_fp4_bytes,
-                "hidden_states_scale": hidden_states_scale_fp4_bytes.view(
-                    torch.float8_e4m3fn
-                ).reshape(-1),
+                "hidden_states_scale": hidden_states_scale_fp4_bytes,
             }
         else:  # bf16
             return {
@@ -463,7 +456,7 @@ class FP4Moe(Moe):
                 num_elts_per_sf=16,
             )
             gemm1_scales_fp4_shuffled.append(
-                nvfp4_block_scale_interleave(
+                block_scale_interleave(
                     gemm1_scales_linear_fp4[i]
                     .view(torch.uint8)[
                         permute_sf_indices.to(gemm1_scales_linear_fp4.device)
@@ -490,7 +483,7 @@ class FP4Moe(Moe):
                 num_elts_per_sf=16,
             )
             gemm2_scales_fp4_shuffled.append(
-                nvfp4_block_scale_interleave(
+                block_scale_interleave(
                     gemm2_scales_linear_fp4[i]
                     .view(torch.uint8)[
                         permute_sf_indices.to(gemm2_scales_linear_fp4.device)
@@ -709,6 +702,7 @@ class FP8BlockScaleMoe(Moe):
         routed_scaling = kwargs["routed_scaling"]
         routing_method_type = kwargs["routing_method_type"]
         tile_tokens_dim = kwargs["tile_tokens_dim"]
+        enable_pdl = kwargs.get("enable_pdl")
 
         # Generate block scales and quantize hidden states at runtime
         hidden_states_fp8 = hidden_states_orig.to(torch.float8_e4m3fn)
@@ -738,6 +732,7 @@ class FP8BlockScaleMoe(Moe):
             routing_method_type,
             use_shuffled_weight=static_data["use_shuffled_weight"],
             weight_layout=static_data["weight_layout"],
+            enable_pdl=enable_pdl,
         )
 
         return output.to(torch.float)
@@ -1740,9 +1735,9 @@ def cache_permute_indices():
 @pytest.mark.parametrize(
     "moe_impl",
     [
-        pytest.param(FP4Moe(quant_mode=QuantMode.FP4_NVFP4_NVFP4), id="NvFP4 x NvFP4"),
-        pytest.param(FP4Moe(quant_mode=QuantMode.FP4_MXFP4_MXFP8), id="MxFP4 x MxFP8"),
-        pytest.param(FP4Moe(quant_mode=QuantMode.FP4_MXFP4_Bf16), id="MxFP4 x Bf16"),
+        pytest.param(FP4Moe(quant_mode=QuantMode.FP4_NVFP4_NVFP4), id="NvFP4xNvFP4"),
+        pytest.param(FP4Moe(quant_mode=QuantMode.FP4_MXFP4_MXFP8), id="MxFP4xMxFP8"),
+        pytest.param(FP4Moe(quant_mode=QuantMode.FP4_MXFP4_Bf16), id="MxFP4xBf16"),
         pytest.param(FP8BlockScaleMoe(), id="FP8_Block"),
         pytest.param(FP8PerTensorMoe(), id="FP8_Tensor"),
     ],
@@ -2040,6 +2035,7 @@ def test_moe_quantization_classes(
         routing_method_type=routing_method_type,
         tile_tokens_dim=tile_tokens_dim,
         weight_processing=weight_processing,
+        enable_pdl=True,
     )
 
     # Compare outputs using moe_impl-specific tolerances
